import time
import random
import unittest
import logging
import redis

from src.endemic.db.redis.orm import RedisORM


class TestDbMysqlORM(unittest.TestCase):

    # execute before every test case function run.
    def setUp(self):
        self.__app = 'tests.python.modules.redis'
        self.Class = RedisORM(logging, redis)
        self.Class.config('95.216.153.29', 6379, 'pzqjeLJA7c6NW4EX', 0)

    def test_single(self):
        for item in self.Class.get_all_keys():
            self.Class.delete_value(item)

        self.assertEqual([], self.Class.get_all_keys())
        self.Class.set_value('111', '222')
        self.assertEqual(['111'], self.Class.get_all_keys())

        # Test ttl
        self.Class.set_value('111', '3333', 3)
        time.sleep(1)
        self.assertEqual('3333', self.Class.get_value('111'))
        time.sleep(4)
        self.assertIsNone(self.Class.get_value('111'))

    def test_second(self):
        for item in self.Class.get_all_keys():
            self.Class.delete_value(item)

        data = {
            'first-{}'.format(random.randint(100000, 200000)): str(random.randint(0, 10)),
            'second-{}'.format(random.randint(100000, 200000)): str(random.randint(0, 10)),
            'third-{}'.format(random.randint(100000, 200000)): str(random.randint(0, 10)),
        }

        self.Class.set_hash_from_dict('222', data)
        self.assertEqual(['222'], self.Class.get_all_keys())
        self.assertEqual(data, self.Class.get_hash_by_key('222'))

    def test_third(self):
        for item in self.Class.get_all_keys():
            self.Class.delete_value(item)

        index = 'first-{}'.format(random.randint(100000, 200000))

        data = {
            index: str(random.randint(0, 10)),
            'second-{}'.format(random.randint(100000, 200000)): str(random.randint(0, 10)),
            'third-{}'.format(random.randint(100000, 200000)): str(random.randint(0, 10)),
        }

        self.Class.set_hash_from_dict('222', data)
        self.assertEqual(['222'], self.Class.get_all_keys())
        self.assertEqual(data, self.Class.get_hash_by_key('222'))

        # Remove
        self.Class.remove_key_from_hash('222', index)

        del data[index]

        self.assertEqual(data, self.Class.get_hash_by_key('222'))




if __name__ == '__main__':
    unittest.main()



import sys

import unittest

from src.endemic.logging.email.logger import LoggerSMTP


class TestLoggingSMTPLogger(unittest.TestCase):

    # execute before every test case function run.
    def setUp(self):
        pass

    def test_debug(self):

        logger = LoggerSMTP(
            'smtp.yandex.ru',
            465,
            'mosrem-ru@yandex.ru',
            '080723Zaq1',
            'mosrem-ru@yandex.ru',
            ['mail@endemic.ru'],
            LoggerSMTP.DEBUG,
            'Test Error',
            False
        )

        print('Init {}'.format(self.__class__.__name__))

        logger.debug('123')

        # try:
        #     x = 1 / 0
        # except Exception:
        #     logger.error(message='Error x = 1 / 0', exc_info=sys.exc_info())


if __name__ == '__main__':
    unittest.main()



import unittest
import pika

from src.endemic.rabbit.instance import Rabbit
from src.endemic.logging.fluentd.logger import LoggerFluentd


class TestRabbitMessage(unittest.TestCase):
    PROD_RABBITMQ_USER_LOGIN = 'user_modules'
    PROD_RABBITMQ_USER_PASSWORD = '$C573bpM9Rbl'
    PROD_RABBITMQ_HOST = '213.226.68.10'
    PROD_RABBITMQ_PORT = 5672
    PROD_RABBITMQ_EXCHANGE = 'application'
    PROD_RABBITMQ_VIRTUAL_HOST = 'modules'

    # Rabbit
    PROD_SSL_RABBITMQ_USER_LOGIN = 'sbfpgnmw'
    PROD_SSL_RABBITMQ_USER_PASSWORD = 'Oirg-mNfI8bVvdikTpPGm5h139yzWKQi'
    PROD_SSL_RABBITMQ_HOST = 's2r_mq.freshapp-iot.com'
    PROD_SSL_RABBITMQ_PORT = 5671
    PROD_SSL_RABBITMQ_EXCHANGE = 'application'
    PROD_SSL_RABBITMQ_VIRTUAL_HOST = 'prod'

    # execute before every test case function run.
    def setUp(self):
        self.__app = 'tests.python.modules.rabbit.instance'
        self.__logger = LoggerFluentd('test-1', self.__app, 'fluentd', LoggerFluentd.INFO)

    def test_single(self):
        instance = Rabbit(self.__logger,
                          pika,
                          self.PROD_RABBITMQ_HOST,
                          self.PROD_RABBITMQ_PORT,
                          self.PROD_RABBITMQ_USER_LOGIN,
                          self.PROD_RABBITMQ_USER_PASSWORD,
                          self.PROD_RABBITMQ_VIRTUAL_HOST,
                          self.PROD_RABBITMQ_EXCHANGE)

        # instance.queue = 'account.private'
        # instance.callback = callback
        #
        # instance.run()

    def test_single_ssl(self):
        instance = Rabbit(self.__logger,
                          pika,
                          self.PROD_SSL_RABBITMQ_HOST,
                          self.PROD_SSL_RABBITMQ_PORT,
                          self.PROD_SSL_RABBITMQ_USER_LOGIN,
                          self.PROD_SSL_RABBITMQ_USER_PASSWORD,
                          self.PROD_SSL_RABBITMQ_VIRTUAL_HOST,
                          self.PROD_SSL_RABBITMQ_EXCHANGE,
                          True)

        instance.queue = 'account.private'
        instance.callback = callback

        instance.run_blocking()

        for i in range(60000):
            instance.publish_message({'test': True}, 'sensor.data', 10)


    def test_create_queue(self):
        instance = Rabbit(self.__logger,
                          pika,
                          self.PROD_RABBITMQ_HOST,
                          self.PROD_RABBITMQ_PORT,
                          self.PROD_RABBITMQ_USER_LOGIN,
                          self.PROD_RABBITMQ_USER_PASSWORD,
                          self.PROD_RABBITMQ_VIRTUAL_HOST,
                          self.PROD_RABBITMQ_EXCHANGE)

        instance.queue = 'test.without.priority'
        instance.exchange = 'application'
        instance.callback = callback

        instance.run()




def callback(ch, method, properties, message):
    print()


if __name__ == '__main__':
    unittest.main()



import unittest
import logging
import pymysql
from functools import wraps

from src.endemic.db.mysql.orm import MysqlORM


def request():
    def wrapper(f):
        @wraps(f)
        def wrapped(self, *args, **kwargs):
            result = []
            [query, values] = f(self, *args, **kwargs)
            with self.mysql_connection.cursor() as cursor:
                try:
                    cursor.execute(query, values)
                    result = cursor.fetchall()
                except pymysql.IntegrityError as err:
                    errnum, errmsg = err.args
                    if errnum == 1062:
                        print(err)
                    else:
                        raise ValueError(errnum, errmsg)
            self.mysql_connection.commit()
            return result

        return wrapped

    return wrapper


class Mysql(MysqlORM):
    def __init__(self, logger: logging):
        super().__init__(logger)
        self.__mysql_connection = None

    @request()
    def pure_sql_request(self, query=''):
        return super().pure_sql_request(query)

    def close(self):
        self.__mysql_connection.close()

    def run(self):
        self.__mysql_connection = pymysql.connect(
            host=self.host,
            port=self.port,
            user=self.user,
            password=self.password,
            db=self.db_name,
            charset='utf8mb4',
            cursorclass=pymysql.cursors.DictCursor)

    @property
    def mysql_connection(self):
        if not self.__mysql_connection:
            raise ValueError('Not connected')
        return self.__mysql_connection


class TestDbMysqlORM(unittest.TestCase):

    # execute before every test case function run.
    def setUp(self):
        self.__app = 'tests.python.modules.mysql-orm'

    def te1st_single(self):
        self.__orm = Mysql(logging)
        self.__orm.config('51.38.127.206', 3306, 'bitweb_user', 'gHCitcOM)9ij789ZZ', 'bitweb')
        self.__orm.run()
        result = self.__orm.pure_sql_request('Select * FROM proxy WHERE 1')
        self.assertLess(10, len(result))


if __name__ == '__main__':
    unittest.main()



from urllib.request import urlopen
from urllib.parse import urlencode

from functools import wraps

from ..interface import LoggerInterface

API = 'https://api.telegram.org/bot{token}/{method}'


def decorator_add_header(title):
    def wrapper(f):
        @wraps(f)
        def wrapped(self, text: str):
            return f(self, '{emoji} <b>[{title}]</b>\n{text}'.format(emoji=title,
                                                                     title=self.message_title,
                                                                     text=text.replace('<', '&lt;')))

        return wrapped

    return wrapper


# todo: add button

class LoggerTelegram(LoggerInterface):

    # threads - number of pool threads
    def __init__(
            self,
            token,
            chat_id,
            title: str = '',
            level: str = LoggerInterface.DEBUG,
    ):
        self.__chat_id = chat_id
        self.__token = token
        self.__message_title = title
        self.__logging_level = level

    @decorator_add_header('👷')
    def debug(self, text, tag=None):
        self.__send_message(text) if self.__logging_level <= LoggerInterface.DEBUG else None

    @decorator_add_header('💚')
    def info(self, text, tag=None):
        self.__send_message(text) if self.__logging_level <= LoggerInterface.INFO else None

    @decorator_add_header('🔶')
    def warning(self, text, tag=None):
        self.__send_message(text) if self.__logging_level <= LoggerInterface.WARNING else None

    @decorator_add_header('🔴')
    def error(self, text, tag=None):
        self.__send_message(text) if self.__logging_level <= LoggerInterface.ERROR else None

    @decorator_add_header('🔥')
    def exception(self, text, tag=None):
        self.__send_message(text) if self.__logging_level <= LoggerInterface.ERROR else None

    @decorator_add_header('‼️')
    def critical(self, text, tag=None):
        self.__send_message(text) if self.__logging_level <= LoggerInterface.CRITICAL else None

    def __send_message(self, message, notify=True):
        return self.__telegram('sendMessage', {
            'text': message,
            'chat_id': self.__chat_id,
            'parse_mode': 'HTML',
            'disable_notification': not notify})

    def __telegram(self, name, data):
        urlopen(f"https://api.telegram.org/bot{self.__token}/{name}?{urlencode(data)}")

    @property
    def message_title(self) -> str:
        return self.__message_title

    @message_title.setter
    def message_title(self, value):
        self.__message_title = value



import logging
import logging.handlers
from typing import Tuple, Union, List, Set
from abc import ABC

from logging.handlers import SMTPHandler


from ..interface import LoggerInterface


class SSLSMTPHandler(SMTPHandler):

    def __init__(self,
                 mailhost: Union[str, Tuple[str, int]],
                 fromaddr: str,
                 toaddrs: List[str],
                 subject: str,
                 credentials: Tuple[str, str],
                 secure,
                 start_ssl: bool = False
                 ):

        super().__init__(mailhost, fromaddr, toaddrs, subject, credentials,secure)
        self.__start_ssl = start_ssl

    def emit(self, record):
        """
        Overwrite the logging.handlers.SMTPHandler.emit function with SMTP_SSL.
        Emit a record.
        Format the record and send it to the specified addressees.
        """
        try:
            import smtplib
            from email.message import EmailMessage
            import email.utils

            smtp = smtplib.SMTP_SSL(
                self.mailhost,
                self.mailport if self.mailport else smtplib.SMTP_PORT)

            smtp.set_debuglevel(1)

            msg = EmailMessage()
            msg['From'] = self.fromaddr
            msg['To'] = ','.join(self.toaddrs)
            msg['Subject'] = self.getSubject(record)
            msg['Date'] = email.utils.localtime()
            msg.set_content(self.format(record))

            if self.username:
                if self.secure is not None:
                    smtp.ehlo()
                    if self.__start_ssl:
                        smtp.starttls(*self.secure)
                        smtp.ehlo()
                smtp.login(self.username, self.password)
            smtp.send_message(msg)
            smtp.quit()
        except Exception:
            self.handleError(record)


class LoggerSMTP(LoggerInterface, ABC):

    def __init__(
            self,
            smtp_host: str,
            smtp_port: int,
            smtp_login: str,
            smtp_pass: str,
            email_from: str,
            emails_to: list,
            level: str = LoggerInterface.DEBUG,
            subject: str = 'Error',
            smtp_start_ssl: bool = False
    ):
        self.__log = logging
        self.__logging_level = level

        self.__smtp = [smtp_host, smtp_port, smtp_login, smtp_pass, smtp_start_ssl]

        self.__email_from = email_from
        self.__email_to = emails_to

        self.__subject = subject

        self.__settings()

    def __repr__(self):
        return self.__log

    def __settings(self):
        print('Setting')
        logging.basicConfig(
            format='%(asctime)s - %(levelname)s - %(message)s',
            level=self.__logging_level
        )

        self.__log = logging.getLogger(__name__)

        smtp_handler = SSLSMTPHandler(
            mailhost=(self.__smtp[0], int(self.__smtp[1])),
            fromaddr=self.__email_from,
            toaddrs=self.__email_to,
            subject=self.__subject,
            credentials=(self.__smtp[2], self.__smtp[3]),
            secure=())

        self.__log.addHandler(smtp_handler)

    def start_ssl(self) -> bool:
        print('RRRR')
        return bool(self.__smtp[4])



import inspect
from functools import wraps
from .constant import *

"""
Rabbitmq message example
{
    "project_flow": [
        {
            "task": "......",
            "link": "......",
            'current-iteration': 1,
            'next-task-queue': "...",
            'next-task-priority': "..."
        },
        {
            "task": "......",
            "link": "......",
            'current-iteration': 1,
            'next-task-queue': "...",
            'next-task-priority': "..."
        },
        {
            "task": "......",
            "link": "......",
            'current-iteration': 1,
            'next-task-queue': "...",
            'next-task-priority': "..."
        },
        ....
    ],
    "message-created": "2021-01-02 15:53:25",
    "link": "0efa7304-dc86-4fd3-95fc-9c9d2da7ae5b",
    "data: []
}
"""


def decorator_check_exits_variable(part: str, name: str, is_raise=False):
    def wrapper(f):
        @wraps(f)
        def wrapped(self):

            if not self.message:
                raise ValueError('Message not exists')

            body = self.current_task if part == PART_TASK else self.message

            if name not in body:
                if is_raise:
                    raise ValueError('Variable "{}" not in {}'.format(name, part))
                return None

            return f(self)

        return wrapped

    return wrapper


class Parser:

    def __init__(self):
        self.__message = None
        self.__current_queue = None
        self.__current_priority = 1

        self.__tasks_all = []

    @property
    @decorator_check_exits_variable(PART_MESSAGE, LINK, False)
    def link_message(self) -> str:
        return self.message[LINK] if self.message[LINK] else ''

    @property
    @decorator_check_exits_variable(PART_TASK, LINK, True)
    def link_task(self) -> str:
        return self.current_task.get(LINK, '')

    @property
    @decorator_check_exits_variable(PART_TASK, TASK, False)
    def task_title(self) -> str:
        return self.current_task[TASK] if TASK in self.current_task else None

    @property
    @decorator_check_exits_variable(PART_TASK, NEXT_TASK_QUEUE, False)
    def next_task_queue(self):
        return self.current_task[NEXT_TASK_QUEUE] if NEXT_TASK_QUEUE in self.current_task else ''

    @property
    @decorator_check_exits_variable(PART_TASK, NEXT_TASK_PRIORITY, False)
    def next_task_priority(self):
        return self.current_task[NEXT_TASK_PRIORITY] if NEXT_TASK_PRIORITY in self.current_task else ''

    @property
    @decorator_check_exits_variable(PART_TASK, CURRENT_ITERATION, False)
    def iteration(self):
        return self.current_task[CURRENT_ITERATION] if CURRENT_ITERATION in self.current_task else 0

    @property
    def queue(self):
        if not self.__current_queue:
            method_name = inspect.currentframe().f_code.co_name
            raise ValueError('"{}" is empty'.format(method_name))
        return self.__current_queue

    @property
    def priority(self):
        if not self.__current_priority:
            method_name = inspect.currentframe().f_code.co_name
            raise ValueError('"{}" is empty'.format(method_name))
        return self.__current_priority

    @property
    def message(self):
        if not self.__message:
            method_name = inspect.currentframe().f_code.co_name
            raise ValueError('"{}" is empty'.format(method_name))
        return self.__message

    @property
    def tasks_all(self) -> list:
        return self.__tasks_all

    @property
    def next_tasks(self) -> list:
        return self.__tasks_all[1:]

    @property
    def current_task(self) -> dict:
        return self.__tasks_all[0] if len(self.__tasks_all) else {}

    @property
    def message_created(self) -> str:
        return self.message[MESSAGE_CREATED] if MESSAGE_CREATED in self.message else ''

    @queue.setter
    def queue(self, value: str):
        self.__current_priority = value

    @priority.setter
    def priority(self, value: int):
        self.__current_priority = value

    @message.setter
    def message(self, value: dict):
        self.__message = value
        self.__tasks_all = self.__message[PROJECT_FLOW] if PROJECT_FLOW in self.__message else []



import logging
import uuid
from time import time_ns
from logging import Logger
from fluent import handler
import traceback
from ..interface import LoggerInterface

custom_format = {
    'host': '%(hostname)s',
    'type': '%(levelname)s',
    'stack_trace': '%(exc_text)s',
    'time': '%(created)s',
    'stack_info': '%(stack_info)s',
}


class LoggerFluentd(LoggerInterface):

    # threads - number of pool threads
    def __init__(
            self,
            instance: str,
            log_name: str,
            logger_tag: str = 'fluentd',
            level: str = LoggerInterface.DEBUG,
            host: str = '',
            port: int = 0
    ):

        super().__init__()
        self.__log = Logger

        self.__instance_name = instance
        self.__logging_name = log_name

        self.__min_info = []
        self.__min_warning = []

        self.__link = ''
        self.__timer = 0

        self.__host = host
        self.__port = port
        self.__level = level
        self.__logger_tag = logger_tag

        self.__setting(host, port, level, logger_tag, log_name)

    def __setting(self, host, port, level, logger_tag, log_name):
        """Setting logger and console output"""
        logging.basicConfig(format='%(asctime)s | %(levelname)-8s | %(message)s',
                            datefmt='%d/%m %H:%M')

        if self.__min_info:
            for i in self.__min_info:
                logging.getLogger(i).setLevel(logging.INFO)

        if self.__min_warning:
            for i in self.__min_warning:
                logging.getLogger(i).setLevel(logging.WARNING)

        if host and port:
            print('Has external connection')
            h = handler.FluentHandler(log_name, host=host, port=port)
        else:
            h = handler.FluentHandler(log_name)

        formatter = handler.FluentRecordFormatter(custom_format)
        h.setFormatter(formatter)

        self.__log = logging.getLogger(logger_tag)
        self.__log.setLevel(level)

        if host and port:
            self.__log.addHandler(h)

    # todo: test
    def log_name(self, log_name: str):
        self.__setting(self.__host, self.__port, self.__level, self.__logger_tag, log_name)

    # todo: test
    def instance_name(self, instance_name: str):
        self.__instance_name = instance_name

    def debug(self, message, class_name: str = None, method_name: str = None, tag: str = None, variables: dict = None,
              exc_info=None):
        self.__log.debug({**{self.MSG: message, self.TAG: tag}, **self.custom(variables, class_name, method_name)})

    def info(self, message, class_name: str = None, method_name: str = None, tag: str = None, variables: dict = None,
             exc_info=None):
        self.__log.info({**{self.MSG: message, self.TAG: tag}, **self.custom(variables, class_name, method_name)})

    def warning(self, message, class_name: str = None, method_name: str = None, tag: str = None, variables: dict = None,
                exc_info=None):
        self.__log.warning({**{self.MSG: message, self.TAG: tag}, **self.custom(variables, class_name, method_name)})

    def error(self, message, class_name: str = None, method_name: str = None, tag: str = None, variables: dict = None,
              exc_info=None):
        self.__log.error({**{self.MSG: message, self.TAG: tag}, **self.custom(variables, class_name, method_name)})

    def critical(self, message, class_name: str = None, method_name: str = None, tag: str = None,
                 variables: dict = None, exc_info=None):
        self.__log.critical({**{self.MSG: message, self.TAG: tag}, **self.custom(variables, class_name, method_name)})

    def exception(self, message=None, class_name: str = None, method_name: str = None, tag: str = None,
                  variables: dict = None, exc_info=None):
        self.__log.exception({**{self.MSG: '[Ex]: {}'.format(message), self.TAG: tag},
                              **self.custom(variables, class_name, method_name)})

    @property
    def func(self):
        return traceback.extract_stack(None, 2)[0][2]

    def custom(self, variables, class_name, method_name) -> dict:
        """Additional custom fields to exist custom format"""
        append = variables if variables else {}
        duration = {'duration': self.timer} if self.__timer else {}

        return {
            **append,
            **duration,
            **{
                'class': class_name,
                'method': method_name,
                'link': self.link,
                'instance': self.__instance_name
            }
        }

    #
    #   Setters/getters ----------------------------------------------------------------------------
    #

    @property
    def link(self) -> str:
        if not self.__link:
            self.__link = str(uuid.uuid4())
        return self.__link

    @link.setter
    def link(self, value: str):
        self.__timer = time_ns()
        self.__link = value

    @property
    def name(self) -> str:
        return self.__logging_name

    @property
    def timer(self) -> float:
        return (time_ns() - self.__timer) * 1.0e-9

    @property
    def min_level_info(self) -> list:
        return self.__min_info

    @min_level_info.setter
    def min_level_info(self, value: str):
        self.__min_info.append(value)

    @property
    def min_level_warning(self) -> list:
        return self.__min_warning

    @min_level_warning.setter
    def min_level_warning(self, value: str):
        self.__min_warning.append(value)



from ..logging.interface import LoggerInterface


class GoogleSheetAPI:
    SCOPES = ['https://www.googleapis.com/auth/spreadsheets', 'https://www.googleapis.com/auth/drive']

    def __init__(self, logger: LoggerInterface, pygsheets_class):
        self.__logger = logger

        self.__pygsheets = pygsheets_class

        self.__google = None
        self.__google_credential_json = None
        self.__drive_folder_id = None

    def connect(self):

        if not self.google_credential_file_json:
            raise ValueError('Credential file path is None')

        self.__google = self.__pygsheets.authorize(service_file=self.__google_credential_json)

        return True

    def create_spreadsheet(self, title):
        self.__logger.debug('Create spreadsheet')

        return self.google.create(title, None, self.drive_folder_id)

    def list_sheets_ids(self):

        return self.google.spreadsheet_ids()

    def list_sheets_titles(self):

        return self.google.spreadsheet_titles()

    def list_sheets_meta(self):

        result_list = self.google.drive.spreadsheet_metadata()

        result = []
        if self.drive_folder_id:
            for item in result_list:
                if 'parents' in item:
                    if self.drive_folder_id in item['parents']:
                        result.append({'id': item['id'], 'name': item['name']})
            return result
        return result_list

    def share_doc(self, sh, email):
        self.__logger.debug('Share spreadsheet')

    def open_spreadsheet_by_id(self, key_id):
        self.__logger.debug('Open spreadsheet by key ')
        try:
            return self.google.open_by_key(key_id)
        except self.__pygsheets.exceptions.SpreadsheetNotFound:
            return None

    def open_spreadsheet_by_title(self, title):
        self.__logger.debug('Open spreadsheet by title ')

        spread_id = self.search_spreadsheet_by_title(title)
        self.__logger.debug('Key id is {}'.format(spread_id))
        if spread_id:
            return self.open_spreadsheet_by_id(spread_id)
        return None

    def search_spreadsheet_by_title(self, title):

        self.__logger.debug('Search spreadsheet by title')

        spread_id = None
        if self.drive_folder_id:
            spreads = self.list_sheets_meta()
            self.__logger.debug('List in folder {}'.format(spreads))
            for item in spreads:
                if title == item['name']:
                    spread_id = item['id']
                    break

        return spread_id

    def find_worksheet_by_title(self, sh, title):
        try:
            return self.google.spreadsheet_cls.worksheet_by_title(sh, title)
        except self.__pygsheets.exceptions.WorksheetNotFound:
            return None

    def create_worksheet_by_name(self, sh, title, cols, rows):
        try:
            return self.google.spreadsheet_cls.add_worksheet(sh, title, rows, cols)
        except self.__pygsheets.exceptions.WorksheetNotFound:
            self.__logger.exception('Worksheet not created')
            return False

    def find_cell_by_value(self, sheet, value):
        self.__logger.debug('find cell by value')

    def add_value_cell_by_coordinates(self, sheet, row, col, value):
        self.__logger.debug('add value to cell')
        try:
            sheet.update_value((row, col), value)
        except Exception:
            return False
        return True

    def merge_cells(self, sheet, row_start, row_end, col_start, col_end, merge_type='MERGE_ROWS'):
        self.__logger.debug('merge cell')
        address_start = self.__pygsheets.worksheet.Address((row_start, col_start))
        address_end = self.__pygsheets.worksheet.Address((row_end, col_end))
        print(address_start, address_end)

        datarange = self.__pygsheets.worksheet.DataRange(worksheet=sheet, start=address_start, end=address_end)
        datarange.merge_cells(merge_type)

    def set_cell_width(self, sheet, column_start, column_end, width):
        self.__logger.debug('Change columns width')
        sheet.adjust_column_width(column_start, column_end, width)

    @property
    def google_credential_file_json(self) -> str:
        return self.__google_credential_json

    @google_credential_file_json.setter
    def google_credential_file_json(self, value: str):
        self.__google_credential_json = value

    @property
    def drive_folder_id(self) -> str:
        return self.__drive_folder_id

    @drive_folder_id.setter
    def drive_folder_id(self, value: str):
        self.__drive_folder_id = value

    @property
    def google(self):
        return self.__google

    @google.setter
    def google(self, value):
        if isinstance(value, self.__pygsheets.client.Client):
            self.__google = value
        else:
            raise ValueError('Not string')



from .fluentd.logger import LoggerFluentd
from .email.logger import LoggerSMTP
from .telegram.logger import LoggerTelegram

__all__ = (  # Keep this alphabetically ordered
    'LoggerFluentd',
    'LoggerSMTP',
    'LoggerTelegram'
)



TASK = 'task'
DATA = 'data'
LINK = 'link'
TARGET = 'target'

PROJECT_FLOW = 'project-flow'
CURRENT_ITERATION = 'current-iteration'
MESSAGE_CREATED = 'message-created'

NEXT_TASK_QUEUE = 'next-task-queue'
NEXT_TASK_PRIORITY = 'next-task-priority'

PART_TASK = 'task'
PART_MESSAGE = 'message'



import random
import unittest
import time

from src.endemic.logging.fluentd.logger import LoggerFluentd
import logging

FLUENTD_APP_HOST = '88.99.81.198'
FLUENTD_APP_PORT = 24224


class TestLoggingFluentLogger(unittest.TestCase):

    # execute before every test case function run.
    def setUp(self):
        self.__app = 'application.shai.modules.logging.fluentd.logger'
        pass

    def test_debug(self):
        print('Init {}'.format(self.__class__.__name__))

        log = LoggerFluentd('test-1',
                            self.__app,
                            'fluentd',
                            LoggerFluentd.DEBUG,
                            FLUENTD_APP_HOST,
                            FLUENTD_APP_PORT)
        self.check_instance(log)

        log.link = '1'

        log.debug('Test {}'.format(random.randint(1, 1000)),
                  self.__class__.__name__, log.func, 'test-tag', {'field1': 1, 'field2': 2})

        for i in range(1):
            try:
                a = 2 / 0
            except Exception as e:
                log.exception(repr(e))
                log.critical('Critical error')
                pass

        print(log.timer)

    def test_info(self):
        print('Init {}'.format(self.__class__.__name__))

        log2 = LoggerFluentd('test-2', self.__app)

        log2.info('Test {}'.format(random.randint(1, 1000)),
                  self.__class__.__name__, log2.func, 'test-tag', {'field1': 1, 'field2': 2})

        log2.warning('Test {}'.format(random.randint(1, 1000)),
                     self.__class__.__name__, log2.func, 'test-tag', {'field1': 1, 'field2': 2})

    def test_exception(self):
        print('Init {}'.format(self.__class__.__name__))

        log2 = LoggerFluentd('test-3', self.__app)

        try:
            a = 2 / 0
        except:
            log2.exception('error test-3')
            log2.critical('Critical error')
            pass

    def check_instance(self, logger: logging):
        pass

    def test_multiple_loggers(self):
        log2 = LoggerFluentd('test-2', self.__app, 'fluentd.2', LoggerFluentd.INFO)
        log1 = LoggerFluentd('test-1', self.__app, 'fluentd.1', LoggerFluentd.DEBUG)

        log1.debug('DEBUG')
        log1.info('INFO')

        log2.debug('DEBUG')
        log2.info('INFO')







if __name__ == '__main__':
    unittest.main()



import random
import unittest

from src.endemic.logging.telegram.logger import LoggerTelegram


class TestLoggingTelegramLogger(unittest.TestCase):

    # execute before every test case function run.
    def setUp(self):
        pass

    def test_debug(self):

        log = LoggerTelegram(
            '1047178763:AAEZpCTDn53WGWIDBQ5K4Tg2fBW4enrlweg',
            '-435754055',
            'test.logging',
            LoggerTelegram.INFO
        )

        print('Init {}'.format(self.__class__.__name__))

        log.debug('123 ? < > * _ ')
        log.info('123 ? < > * _ ')
        log.warning('123 ? < > * _ ')
        # log.error('123 ? < > * _ ')
        # log.exception('123 ? < > * _ ')
        # log.critical('123 ? < > * _ ')


if __name__ == '__main__':
    unittest.main()



import logging


class RedisORM:

    def __init__(self, logger: logging, redis):
        self.__logger = logger
        self.__host = None
        self.__port = None
        self.__db_index = 0
        self.__password = None
        self.__connection = None
        self.__redis = redis

    def config(self, host=None, port=None, password=None, db_index=None):
        self.__host = host
        self.__port = port
        self.__password = password
        self.__db_index = db_index
        self.__connection = self.__redis.StrictRedis(host=self.__host,
                                                     port=self.__port,
                                                     password=self.__password,
                                                     decode_responses=True,
                                                     db=self.__db_index)

    def set_value(self, key: str, value: str, ttl=0):
        self.__connection.set(key, value) if not ttl else self.__connection.set(key, value, ex=ttl)
        return True

    def set_hash_from_dict(self, key: str, data: dict):
        p = self.__connection.pipeline()
        p.hset(key, mapping=data)
        p.execute()

    def delete_value(self, key: str):
        self.__connection.delete(key)

    def get_value(self, key: str) -> str:
        return self.__connection.get(key)

    def get_value_by_index(self, key: str, index: str) -> str:
        return self.__connection.hget(key, index)

    def get_all_keys(self) -> list:
        return self.__connection.keys()

    def get_hash_by_key(self, key: str) -> dict:
        return self.__connection.hgetall(key)

    def get_used_memory(self) -> int:
        return self.__connection.info()['used_memory']

    def get_db_size(self) -> int:
        return self.__connection.dbsize()

    def remove_key_from_hash(self, index: str, key: str):
        p = self.__connection.pipeline()
        p.hdel(index, key)
        p.execute()

    def remove_all_data_use_really_carefully(self):
        self.__connection.flushall()

    @property
    def host(self) -> str:
        return self.__host

    @property
    def port(self) -> int:
        return self.__port

    @property
    def db_index(self) -> int:
        return self.__db_index

    @property
    def connection(self):
        return self.__connection

    @property
    def __pipeline(self):
        return self.__connection.pipeline()



import unittest

from src.endemic.rabbit import Parser


class TestPublicAccount(unittest.TestCase):

    # execute before every test case function run.
    def setUp(self):
        self.__app = 'tests.python.modules.message-parser'
        self.__parser = Parser()

    def test_single(self):
        print(self.__app)

        self.__parser.message = self.__example_message()

        print(self.__parser.tasks_all)

        project_flow = self.__example_message()["project-flow"]

        self.assertEqual(6, len(self.__parser.tasks_all))
        self.assertEqual(project_flow[0]['task'], self.__parser.task_title)
        self.assertEqual(project_flow[0]['link'], self.__parser.link_task)
        self.assertEqual(project_flow[0]['current-iteration'], self.__parser.iteration)
        self.assertEqual(project_flow[0]['next-task-queue'], self.__parser.next_task_queue)
        self.assertEqual(project_flow[0]['next-task-priority'], self.__parser.next_task_priority)

        self.assertEqual('0efa7304-dc86-4fd3-95fc-9c9d2da7ae5b', self.__parser.link_message)

    def __example_message(self):
        return {
            "project-flow": [
                {
                    "task": "task-1",
                    "link": "link-1",
                    'current-iteration': 121,
                    'next-task-queue': "next-q-1",
                    'next-task-priority': "next-t-1"
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
            ],
            "message-created": "2021-01-02 15:53:25",
            "link": "0efa7304-dc86-4fd3-95fc-9c9d2da7ae5b",
            "data": []
        }


if __name__ == '__main__':
    unittest.main()



import logging
import json


class MysqlORM:

    def __init__(self, logger: logging):
        self.logging = logger
        self.__host = None
        self.__port = None
        self.__user = None
        self.__password = None
        self.__db_name = None

    def config(self, host=None, port=None, user=None, password=None, db_name=None):
        self.__host = host
        self.__port = port
        self.__user = user
        self.__password = password
        self.__db_name = db_name

    @staticmethod
    def pure_sql_request(query, binds=None):
        return [query, binds]

    def insert_into_table_on_duplicate_ignore(self, table, data, insert_keys):
        export_keys = list(insert_keys.keys())

        temp = []
        values = []
        for elem in export_keys:
            try:
                if isinstance(data[elem], int):
                    temp.append('%s')
                    values.append(int(data[elem]))
                else:
                    temp.append('%s')
                    values.append(str(data[elem]))
            except KeyError:
                del insert_keys[elem]
                pass

        export_fields = list(insert_keys.values())

        query = 'INSERT INTO {0} ({1}) VALUES ({2}) ON DUPLICATE KEY UPDATE id=VALUES(id)'.format(
            table,
            self.__create_string_from_list_of_values(export_fields),
            self.__create_string_from_list_of_values(temp)
        )

        return [query, values]

    def insert_into_table_increase_field(self, table, data, field, direction='+'):
        export_keys = list(data.keys())

        temp = []
        values = []
        for elem in export_keys:
            if isinstance(data[elem], int):
                temp.append('%s')
                values.append(int(data[elem]))
            else:
                temp.append('%s')
                values.append(str(data[elem]))

        query = 'INSERT INTO ' + str(table) + ' (' + self.__create_list_of_keys(export_keys) + ') VALUES (' \
                + self.__create_string_from_list_of_values(
            temp) + ') ON DUPLICATE KEY UPDATE ' + field + ' = ' + field + direction + '1'

        return [query, values]

    # SQL query example: SELECT * FROM {table} WHERE {field} IN ({list(data)} Order By {order} {direction} Limit {limit})
    # table - table name
    # data - dict
    # order - field name to order (string)
    # direction - ASC/DESC
    # limit - limit in format string 'start,number'
    #
    @staticmethod
    def get_row_form_table_by_field_array(table, data, field, order=None, direction='ASC', limit=None):
        format_strings = ','.join(['%s'] * len(data))
        query = 'SELECT * FROM ' + str(table) + ' WHERE ' + str(field) + ' in (%s) ' % format_strings

        if order:
            query += ' ORDER BY {} {}'.format(order, direction)

        if limit:
            query += ' LIMIT {}'.format(limit)

        return [query, tuple(data)]

    # SQL query example: SELECT * FROM {table} WHERE {field} > (value} Order By {order} {direction} Limit {limit})
    # table - table name
    # data - dict
    # order - field name to order (string)
    # direction - ASC/DESC
    # limit - limit in format string 'start,number'
    #
    @staticmethod
    def get_row_form_table_greater_then_field_array(table, value, field, order=None, direction='ASC', limit=None):
        query = 'SELECT * FROM ' + str(table) + ' WHERE ' + str(field) + ' > (%s) '

        if order:
            query += ' ORDER BY {} {}'.format(order, direction)

        if limit:
            query += ' LIMIT {}'.format(limit)

        return [query, value]

    def update_table_all_values_by_key(self, table, data, insert_keys):
        export_keys = list(insert_keys.keys())

        temp = []
        values = []
        keys = []

        for elem in export_keys:
            if elem in data:
                if isinstance(data[elem], bool):
                    temp.append('%s')
                    values.append(bool(data[elem]))
                    keys.append(str(insert_keys[elem]))
                elif isinstance(data[elem], dict):
                    temp.append('%s')
                    values.append(str(json.dumps(data[elem])))
                    keys.append(str(insert_keys[elem]))
                elif not data[elem]:
                    temp.append('%s')
                    values.append(None)
                    keys.append(str(insert_keys[elem]))
                else:
                    temp.append('%s')
                    values.append(str(data[elem]))
                    keys.append(str(insert_keys[elem]))

        if not ', '.join(['{1}=values({1})'.format(value, value) for (value) in keys]):
            return

        query = 'INSERT INTO ' + str(table) + ' (' + self.__create_string_from_list_of_values(keys) + ') VALUES (' \
                + self.__create_string_from_list_of_values(temp) + ') ON DUPLICATE KEY UPDATE ' + \
                ', '.join(['{1}=values({1})'.format(value, value) for (value) in keys])

        return [query, values]

    @staticmethod
    def delete_from_table(table, field, value):
        query = 'DELETE FROM ' + str(table) + ' WHERE ' + str(field) + ' = %s'
        return [query, value]

    @staticmethod
    def select_row_by_value_between_dates(table, field, value, date_field, date_lower, date_higher):

        query = 'SELECT * FROM ' + str(table) + ' WHERE ' + str(field) + ' in (%s) ' + \
                'AND ' + str(date_field) + ' > %s AND ' + str(date_field) + ' < %s'

        values = [value, date_lower, date_higher]

        return [query, tuple(values)]

    @staticmethod
    def __create_string_from_list_of_values(list_input):
        string_value = ','.join(str(x) for x in list_input)
        return string_value

    @property
    def user(self) -> str:
        return self.__user

    @property
    def host(self) -> str:
        return self.__host

    @property
    def port(self) -> str:
        return self.__port

    @property
    def password(self) -> str:
        return self.__password

    @property
    def db_name(self) -> str:
        return self.__db_name



import logging
from abc import ABCMeta, abstractmethod


class LoggerInterface:

    def __init__(self):
        pass

    MSG = 'message'
    TAG = 'tag'

    DEBUG = logging.DEBUG
    INFO = logging.INFO
    WARNING = logging.WARNING
    ERROR = logging.ERROR
    EXCEPTION = logging.ERROR
    CRITICAL = logging.CRITICAL

    __metaclass__ = ABCMeta

    @abstractmethod
    def info(self, message, class_name: str = None, method_name: str = None, tag: str = None, variables: dict = None,
             exc_info=None):
        pass

    @abstractmethod
    def error(self, message, class_name: str = None, method_name: str = None, tag: str = None, variables: dict = None,
              exc_info=None):
        pass

    @abstractmethod
    def critical(self, message, class_name: str = None, method_name: str = None, tag: str = None,
                 variables: dict = None, exc_info=None):
        pass

    @abstractmethod
    def debug(self, message, class_name: str = None, method_name: str = None, tag: str = None, variables: dict = None,
              exc_info=None):
        pass

    @abstractmethod
    def exception(self, message, class_name: str = None, method_name: str = None, tag: str = None,
                  variables: dict = None, exc_info=None):
        pass

    @abstractmethod
    def warning(self, message, class_name: str = None, tag: str = None, variables: dict = None, method_name: str = None,
                exc_info=None):
        pass

    def log_name(self, log_name: str):
        pass

    def instance_name(self, instance_name: str):
        pass

    @property
    @abstractmethod
    def link(self) -> str:
        return ''

    @link.setter
    @abstractmethod
    def link(self, value: str):
        pass

    @property
    @abstractmethod
    def func(self) -> str:
        return ''

    @property
    @abstractmethod
    def timer(self) -> float:
        return 0.0



import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name='endemic',
    version='0.1.18',
    description='Endemic misc class',
    long_description=README,
    long_description_content_type="text/markdown",
    url="",
    author="Dima Denisov",
    author_email='python@endemic.ru',
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
    ],
    include_package_data=True,
    install_requires=['pymysql', 'fluent-logger'],
    package_dir={'': 'src'},
    packages=find_packages(where='src'),  # Required
)



from datetime import datetime
from .constant import *


class Message:
    def __init__(self):
        self.__link = None

    def default(self, project_flow: list, data: list) -> dict:
        return {
            PROJECT_FLOW: project_flow,
            MESSAGE_CREATED: datetime.now().strftime('%Y-%m-%dT%H:%M:%S.000000Z'),
            LINK: self.__link,
            DATA: data if data else []
        }

    @property
    def link(self):
        return self.__link

    @link.setter
    def link(self, value: str):
        self.__link = value



import unittest
from datetime import datetime

from src.endemic.rabbit.constant import *
from src.endemic.rabbit.parser import Parser


class TestRabbitMessage(unittest.TestCase):

    # execute before every test case function run.
    def setUp(self):
        self.__app = 'tests.python.modules.rabbit.message'
        self.Parser = Parser()

    def test_single(self):
        message = self.__example_message()
        self.Parser.message = message

        self.assertEqual(message[PROJECT_FLOW][0], self.Parser.current_task)
        self.assertEqual(message[PROJECT_FLOW][0][TASK], self.Parser.task_title)
        self.assertEqual(5, len(self.Parser.next_tasks))
        self.assertEqual('0efa7304-dc86-4fd3-95fc-9c9d2da7ae5b', self.Parser.link_message)
        self.assertEqual('2021-01-02 15:53:25', self.Parser.message_created)

    def test_empty_message(self):
        message = {'test': True}
        self.Parser.message = message

        self.assertEqual(None, self.Parser.task_title)
        self.assertEqual(0, len(self.Parser.next_tasks))
        self.assertEqual(None, self.Parser.link_message)
        self.assertEqual('', self.Parser.message_created)

    @staticmethod
    def __example_message():
        return {
            "project-flow": [
                {
                    "task": "task-1",
                    "link": "link-1",
                    'current-iteration': 121,
                    'next-task-queue': "next-q-1",
                    'next-task-priority': "next-t-1"
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
            ],
            "message-created": "2021-01-02 15:53:25",
            "link": "0efa7304-dc86-4fd3-95fc-9c9d2da7ae5b",
            "data": []
        }


if __name__ == '__main__':
    unittest.main()



import unittest
from datetime import datetime

from src.endemic.rabbit.message import Message


class TestRabbitMessage(unittest.TestCase):

    # execute before every test case function run.
    def setUp(self):
        self.__app = 'tests.python.modules.rabbit.message'
        self.__message = Message()

    def test_single(self):
        print(self.__app)

        message = self.__example_message()
        self.__message.link = message['link']

        result = self.__message.default(message['project-flow'], [123])
        self.assertEqual(self.__message.link, result['link'])
        self.assertEqual([123], result['data'])
        self.assertEqual(datetime.now().strftime('%Y-%m-%dT%H:%M:%S.000000'), result['message-created'])
        self.assertEqual(6, len(result['project-flow']))
        self.assertNotEqual(5, len(result['project-flow']))

    def __example_message(self):
        return {
            "project-flow": [
                {
                    "task": "task-1",
                    "link": "link-1",
                    'current-iteration': 121,
                    'next-task-queue': "next-q-1",
                    'next-task-priority': "next-t-1"
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
                {
                    "task": "......",
                    "link": "......",
                    'current-iteration': 1,
                    'next-task-queue': "...",
                    'next-task-priority': "..."
                },
            ],
            "message-created": "2021-01-02 15:53:25",
            "link": "0efa7304-dc86-4fd3-95fc-9c9d2da7ae5b",
            "data": []
        }


if __name__ == '__main__':
    unittest.main()
